use anyhow::{bail, Context};

use crate::packet::RawPacket;

// TODO eventually rewrite Ssdv in Rust?
// Don't want to use FFI because then segfaults can hurt us
pub fn ssdv_encode(callsign: &str, img: &[u8], img_idx: u8) -> anyhow::Result<Vec<RawPacket>> {
    let (stdout, stderr) = subprocess::Exec::cmd("ssdv")
        .args(&[
            "-e",
            "-c",
            callsign,
            "-n",
            "-q",
            "6",
            "-i",
            &format!("{}", img_idx),
        ])
        .stdin(img.to_vec())
        .communicate()?
        .read()?;

    if let Some(stderr) = stderr {
        if !stderr.is_empty() {
            eprintln!("{}", String::from_utf8_lossy(&stderr));
        }
    }

    let stdout = stdout.context("ssdv no stdout")?;

    if stdout.len() % 256 != 0 {
        bail!("ssdv returned garbage");
    }

    let mut out = vec![];
    let mut cur = [0; 256];
    let mut i = 0;
    for c in stdout {
        cur[i] = c;
        i += 1;
        if i == 256 {
            out.push(RawPacket(cur));
            i = 0;
        }
    }

    Ok(out)
}
